#%%
import numpy as np
from pathlib import Path
import pandas as pd
import re
from random import shuffle


def read_swc(path):
    """Read a single swc file
    
    Arguments:
        path {string} -- path to file
    
    Returns:
        df {pandas dataframe} -- indices, coordinates, and parents of each node
        offset {list of floats} -- offset value of fragment
        color {list of ints} -- color
        cc {int} -- cc value, from file name
        branch {int} -- branch number, from file name
    """
    #check input
    file = open(path,'r')
    line = file.readline()
    if line.find('Generated') == -1: 
        msg = 'Error in reading ' + str(path) + '. Line 1 does not have generation information.'
        raise IOError(msg)

    #read offset
    line = file.readline().split()
    string = 'OFFSET'
    if string not in line: 
        msg = 'Error in reading ' + str(path) + '. Line 2 does not have ' + string
        raise IOError(msg)
    idx = line.index(string)+1
    offset = [float(line[i]) for i in np.arange(idx,idx+3)]

    #read color
    line = file.readline().split()
    string = 'COLOR'
    if string not in line: 
        msg = 'Error in reading ' + str(path) + '. Line 3 does not have ' + string
        raise IOError(msg)
    idx = line.index(string)+1
    line = line[idx]
    line = line.split(',')
    color = [float(line[i]) for i in np.arange(len(line))]

    #check name
    line = file.readline().split()
    string = 'NAME'
    if string not in line: 
        msg = 'Error in reading ' + str(path) + '. Line 4 does not have ' + string
        raise IOError(msg)
    idx = line.index(string)+1
    name = line[idx]
    name = re.split('_|-|\.',name)
    idx = name.index('cc')+1
    cc = int(name[idx])
    idx = name.index('branch')+1
    branch = int(name[idx])

    #read coordinates
    df = pd.read_table(path,names=['sample','structure','x','y','z','r','parent'],skiprows=4,delim_whitespace=True)
    return df, offset, color, cc, branch

def read_swc_offset(path):
    df, offset, color, cc,branch = read_swc(path)
    df['x'] = df['x'] + offset[0]
    df['y'] = df['y'] + offset[1]
    df['z'] = df['z'] + offset[2]

    return df, color, cc, branch


def append_df(cumulative, new):
    """Append the dataframe of one fragment to the dataframe of one or more other fragments
    
    Arguments:
        cumulative {pandas dataframe} -- other fragments
        new {pandas dataframe} -- fragment to be appended
    
    
    Returns:
        datafranme -- appended result
    """
    #check cumulative df
    samples = cumulative['sample'].values
    unq = np.unique(samples)
    if len(unq) != cumulative.shape[0]:
        msg = 'cumulative df has ' + str(cumulative.shape[0]) + ' rows but only ' + str(len(unq)) + ' unique sample labels.'
        raise ValueError(msg)
    mx = np.amax(unq)
    sample_new = np.arange(mx+1, mx+1+new.shape[0])
    sample_new = np.append(sample_new,-1)
    sample_old = new['sample'].values
    sample_old = np.append(sample_old,-1)
    parent_old = new['parent'].values

    idxs = [np.argwhere(sample_old == parent_old[i])[0,0] for i in np.arange(len(parent_old))]
    parent_new = [sample_new[idx] for idx in idxs]

    new['sample'] = sample_new[:-1]
    new['parent'] = parent_new
    cumulative = cumulative.append(new)
    return cumulative.reset_index(drop=True)



def read_swc_dir(path, nFiles = 1):
    """Read all swc files in a directory and create a large dataframe of all the results
    
    Arguments:
        path {string} -- path to directory of swc files
    
    Keyword Arguments:
        nFiles {int} -- number of files to process in the directory (default: {1})
    
    Returns:
        pandas dataframe -- dataframe of all the fragment data
    """
    files = list(path.glob('**/*.swc'))[0:nFiles]
    
    for i,file in enumerate(files):
        df, offset, color, cc, branch = read_swc(file)
        df['x'] = df['x'] + offset[0]
        df['y'] = df['y'] + offset[1]
        df['z'] = df['z'] + offset[2]

        df['color0'] = color[0]
        df['color1'] = color[1]
        df['color2'] = color[2]

        df['cc'] = cc
        df['branch'] = branch

        if i==0:
            df_cumulative = df
        else:
            df_cumulative = append_df(df_cumulative, df)

    return df_cumulative

#%%
